@@ -14,14 +14,16 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# The bilinear form a(u, v) and linear form L(v) for
-# Poisson's equation using bilinear elements on bilinear mesh geometry.
-import basix.ufl
-from ufl import (Coefficient, FunctionSpace, Mesh, TestFunction, TrialFunction,
-                 dx, grad, inner)
+"""Complex Poisson demo.
 
-coords = basix.ufl.element("P", "triangle", 2, shape=(2, ))
+The bilinear form a(u, v) and linear form L(v) for
+Poisson's equation using bilinear elements on bilinear mesh geometry.
+"""
+
+import basix.ufl
+from ufl import Coefficient, FunctionSpace, Mesh, TestFunction, TrialFunction, dx, grad, inner
+
+coords = basix.ufl.element("P", "triangle", 2, shape=(2,))
 mesh = Mesh(coords)
 dx = dx(mesh)
 
