@@ -14,18 +14,23 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# This example demonstrates how to create vectors component-wise
+"""Components demo.
+
+This example demonstrates how to create vectors component-wise.
+"""
+
 import basix.ufl
-from ufl import Coefficient, TestFunction, as_vector, dot, dx
+from ufl import Coefficient, FunctionSpace, Mesh, TestFunction, as_vector, dx, inner
 
-element = basix.ufl.element("Lagrange", "tetrahedron", 1, shape=(3, ))
+element = basix.ufl.element("Lagrange", "tetrahedron", 1, shape=(3,))
+domain = Mesh(element)
+space = FunctionSpace(domain, element)
 
-v = TestFunction(element)
-f = Coefficient(element)
+v = TestFunction(space)
+f = Coefficient(space)
 
 # Create vector
 v0 = as_vector([v[0], v[1], 0.0])
 
 # Use created vector in linear form
-L = dot(f, v0) * dx
+L = inner(f, v0) * dx
