@@ -14,20 +14,40 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# Illustration on how to use Conditional to define a source term
+"""Conditional demo.
+
+Illustration on how to use Conditional to define a source term.
+"""
+
 import basix.ufl
-from ufl import (And, Constant, Not, Or, SpatialCoordinate, TestFunction,
-                 conditional, dx, ge, gt, le, lt, triangle)
+from ufl import (
+    And,
+    Constant,
+    FunctionSpace,
+    Mesh,
+    Not,
+    Or,
+    SpatialCoordinate,
+    TestFunction,
+    conditional,
+    dx,
+    ge,
+    gt,
+    inner,
+    le,
+    lt,
+)
 
 element = basix.ufl.element("Lagrange", "triangle", 2)
+domain = Mesh(basix.ufl.element("Lagrange", "triangle", 1, shape=(2,)))
+space = FunctionSpace(domain, element)
 
-v = TestFunction(element)
-g = Constant(triangle)
+v = TestFunction(space)
+g = Constant(domain)
 
-x = SpatialCoordinate(triangle)
-c0 = conditional(le((x[0] - 0.33)**2 + (x[1] - 0.67)**2, 0.015), -1.0, 5.0)
-c = conditional(le((x[0] - 0.33)**2 + (x[1] - 0.67)**2, 0.025), c0, 0.0)
+x = SpatialCoordinate(domain)
+c0 = conditional(le((x[0] - 0.33) ** 2 + (x[1] - 0.67) ** 2, 0.015), -1.0, 5.0)
+c = conditional(le((x[0] - 0.33) ** 2 + (x[1] - 0.67) ** 2, 0.025), c0, 0.0)
 
 t0 = And(ge(x[0], 0.55), le(x[0], 0.95))
 t1 = Or(lt(x[1], 0.05), gt(x[1], 0.45))
@@ -38,4 +58,4 @@ k = conditional(gt(1, 0), g, g + 1)
 
 f = c + t + k
 
-L = v * f * dx
+L = inner(f, v) * dx
