@@ -14,23 +14,41 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# Test form for metadata.
+"""Metadata demo.
+
+Test form for metadata.
+"""
+
 import basix.ufl
-from ufl import Coefficient, TestFunction, TrialFunction, dx, grad, inner
+from ufl import (
+    Coefficient,
+    Constant,
+    FunctionSpace,
+    Mesh,
+    TestFunction,
+    TrialFunction,
+    dx,
+    grad,
+    inner,
+)
 
 element = basix.ufl.element("Lagrange", "triangle", 1)
-vector_element = basix.ufl.element("Lagrange", "triangle", 1, shape=(2, ))
+vector_element = basix.ufl.element("Lagrange", "triangle", 1, shape=(2,))
+domain = Mesh(basix.ufl.element("Lagrange", "triangle", 1, shape=(2,)))
+space = FunctionSpace(domain, element)
+vector_space = FunctionSpace(domain, vector_element)
 
-
-u = TrialFunction(element)
-v = TestFunction(element)
-c = Coefficient(vector_element)
+u = TrialFunction(space)
+v = TestFunction(space)
+c = Coefficient(vector_space)
+c2 = Constant(domain)
 
 # Terms on the same subdomain using different quadrature degree
-a = inner(grad(u), grad(v)) * dx(0, degree=8)\
-    + inner(c, c) * inner(grad(u), grad(v)) * dx(1, degree=4)\
-    + inner(c, c) * inner(grad(u), grad(v)) * dx(1, degree=2)\
+a = (
+    inner(grad(u), grad(v)) * dx(0, degree=8)
+    + inner(c, c) * inner(grad(u), grad(v)) * dx(1, degree=4)
+    + inner(c, c) * inner(grad(u), grad(v)) * dx(1, degree=2)
     + inner(grad(u), grad(v)) * dx(1, degree=-1)
+)
 
-L = v * dx(0, metadata={"precision": 1})
+L = inner(c2, v) * dx(0, metadata={"precision": 1})
