@@ -14,18 +14,23 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# This demo illustrates use of finite element spaces defined over
-# simplicies embedded in higher dimensions
+"""Projection manifold demo.
+
+This demo illustrates use of finite element spaces defined over
+simplicies embedded in higher dimensions.
+"""
+
 import basix.ufl
-from ufl import TestFunctions, TrialFunctions, div, dx, inner
+from ufl import FunctionSpace, Mesh, TestFunctions, TrialFunctions, div, dx, inner
 
 # Define element over this domain
-V = basix.ufl.element("RT", "triangle", 1, gdim=3)
-Q = basix.ufl.element("DG", "triangle", 0, gdim=3)
+V = basix.ufl.element("RT", "triangle", 1)
+Q = basix.ufl.element("DG", "triangle", 0)
 element = basix.ufl.mixed_element([V, Q])
+domain = Mesh(basix.ufl.element("Lagrange", "triangle", 1, shape=(3,)))
+space = FunctionSpace(domain, element)
 
-(u, p) = TrialFunctions(element)
-(v, q) = TestFunctions(element)
+(u, p) = TrialFunctions(space)
+(v, q) = TestFunctions(space)
 
-a = (inner(u, v) + div(u) * q + div(v) * p) * dx
+a = (inner(u, v) + inner(div(u), q) + inner(p, div(v))) * dx
