@@ -52,3 +52,18 @@ def map_facet_points(
         ],
         dtype=np.float64,
     )
+
+
+def map_edge_points(
+    points: npt.NDArray[np.float64], edge: int, cellname: str
+) -> npt.NDArray[np.float64]:
+    """Map points from a reference edge to a physical edge."""
+    geom = np.asarray(basix.geometry(_CellType[cellname]))
+    edge_vertices = [geom[i] for i in basix.topology(_CellType[cellname])[-3][edge]]
+    return np.asarray(
+        [
+            edge_vertices[0] + sum((i - edge_vertices[0]) * j for i, j in zip(edge_vertices[1:], p))
+            for p in points
+        ],
+        dtype=np.float64,
+    )
