@@ -38,13 +38,13 @@ def format_code(code: CodeBlocks) -> tuple[str, str]:
     return code_h, code_c
 
 
-def write_code(code_h, code_c, prefix, output_dir):
+def write_code(code_h: str, code_c: str, prefix: str, output_dir: str) -> None:
     """Write code to files."""
     _write_file(code_h, prefix, ".h", output_dir)
     _write_file(code_c, prefix, ".c", output_dir)
 
 
-def _write_file(output, prefix, postfix, output_dir):
+def _write_file(output: str, prefix: str, postfix: str, output_dir: str) -> None:
     """Write generated code to file."""
     filename = os.path.join(output_dir, prefix + postfix)
     with open(filename, "w") as hfile:
